@@ -166,7 +166,7 @@ class Authentication():
             data['client_secret'] = client_secret
         if pkce_secret:
             raise NotImplementedError
-        res = requests.post(f"{authority_uri}/oauth2/token", data=data)
+        res = requests.post(f"{authority_uri}/oauth2/token", data=data, proxies=self.proxies, verify=self.verify)
         if res.status_code != 200:
             raise AuthenticationException(res.text)
         tokenreply = res.json()
@@ -187,7 +187,7 @@ class Authentication():
             "client_info":1,
             "windows_api_version":"2.0"
         }
-        res = requests.post(f"{authority_uri}/oauth2/token", data=data)
+        res = requests.post(f"{authority_uri}/oauth2/token", data=data, proxies=self.proxies, verify=self.verify)
         if res.status_code != 200:
             raise AuthenticationException(res.text)
         prtdata = res.text
@@ -361,7 +361,7 @@ class Authentication():
         Request server challenge (nonce) to use with a PRT
         """
         data = {'grant_type':'srv_challenge'}
-        res = requests.post('https://login.microsoftonline.com/common/oauth2/token', data=data)
+        res = requests.post('https://login.microsoftonline.com/common/oauth2/token', data=data, proxies=self.proxies, verify=self.verify)
         return res.json()
 
     def get_prt_cookie_nonce(self):
@@ -371,12 +371,14 @@ class Authentication():
         simpler request from the get_srv_challenge function.
         """
         ses = requests.session()
+        ses.proxies = self.proxies
+        ses.verify = self.verify
         params = {
             'resource': self.resource_uri,
             'client_id': self.client_id,
             'response_type': 'code',
             'haschrome': '1',
-            'redirect_uri': 'urn:ietf:wg:oauth:2.0:oob',
+            'redirect_uri': 'https://login.microsoftonline.com/common/oauth2/nativeclient',
             'client-request-id': str(uuid.uuid4()),
             'x-client-SKU': 'PCL.Desktop',
             'x-client-Ver': '3.19.7.16602',
@@ -479,12 +481,14 @@ class Authentication():
                 print('Re-signed PRT cookie using derived key')
 
         ses = requests.session()
+        ses.proxies = self.proxies
+        ses.verify = self.verify
         params = {
             'resource': self.resource_uri,
             'client_id': self.client_id,
             'response_type': 'code',
             'haschrome': '1',
-            'redirect_uri': 'urn:ietf:wg:oauth:2.0:oob',
+            'redirect_uri': 'https://login.microsoftonline.com/common/oauth2/nativeclient',
             'client-request-id': str(uuid.uuid4()),
             'x-client-SKU': 'PCL.Desktop',
             'x-client-Ver': '3.19.7.16602',
@@ -506,11 +510,36 @@ class Authentication():
             ups = urlparse(res.headers['Location'])
             qdata = parse_qs(ups.query)
             return self.authenticate_with_code(qdata['code'][0], params['redirect_uri'])
+        if res.status_code == 302 and 'sso_nonce' in res.headers['Location'].lower():
+            ups = urlparse(res.headers['Location'])
+            qdata = parse_qs(ups.query)
+            if 'sso_nonce' in qdata:
+                nonce = qdata['sso_nonce'][0]
+                print(f'Redirected with new nonce. Old nonce may be expired. New nonce: {nonce}')
         if self.debug:
             with open('roadtools.debug.html','w') as outfile:
                 outfile.write(str(res.headers))
                 outfile.write('------\n\n\n-----')
                 outfile.write(res.content.decode('utf-8'))
+
+        # Try to find SSO nonce in json config
+        startpos = res.content.find(b'$Config=')
+        stoppos = res.content.find(b'//]]></script>')
+        if startpos != -1 and stoppos != -1:
+            jsonbytes = res.content[startpos+8:stoppos-2]
+            try:
+                jdata = json.loads(jsonbytes)
+                try:
+                    error = jdata['strMainMessage']
+                    print(f'Error from STS: {error}')
+                    error = jdata['strAdditionalMessage']
+                    print(f'Additional info: {error}')
+                    error = jdata['strServiceExceptionMessage']
+                    print(f'Exception: {error}')
+                except KeyError:
+                    pass
+            except json.decoder.JSONDecodeError:
+                pass
         print('No authentication code was returned, make sure the PRT cookie is valid')
         print('It is also possible that the sign-in was blocked by Conditional Access policies')
         return False
